package com.baijiayun.groupclassui.window.blackboard;

import android.annotation.TargetApi;
import android.content.Context;
import android.util.AttributeSet;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ScrollView;

public class BlackboardScrollView extends ScrollView {

    private boolean isSingleFingerScrollEnable = true;

    private boolean forbidScroll = false;

    private Runnable scrollerTask;

    private int initialPosition;

    private int newCheckTime = 100;

    private GestureDetector gestureDetector;

    private GestureDetector.OnGestureListener gestureListener;

    private ScrollViewListener scrollViewListener;

    private boolean doubleFingerConfirmed = false;

    private boolean isFling = false;

    public BlackboardScrollView(Context context) {
        this(context, null);
    }

    public BlackboardScrollView(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public BlackboardScrollView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);

        gestureListener = new GestureDetector.OnGestureListener() {
            @Override
            public boolean onDown(MotionEvent e) {
                return false;
            }

            @Override
            public void onShowPress(MotionEvent e) {
                // do nothing
            }

            @Override
            public boolean onSingleTapUp(MotionEvent e) { return false; }

            @Override
            public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX, float distanceY) {
                if (doubleFingerConfirmed) {
                    onTouchEvent(e1);
                    onTouchEvent(e2);
                }
                return false;
            }

            @Override
            public void onLongPress(MotionEvent e) {
                // do nothing
            }

            @Override
            public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) { return false; }
        };

        scrollerTask = () -> {
            int newPosition = getScrollY();

            if (initialPosition - newPosition == 0) {
                float ratio =  (float) newPosition / (float) (getScrollViewHeight());
                if (scrollViewListener != null)
                    scrollViewListener.onScrollStopped(ratio);
            } else {
                initialPosition = getScrollY();
                BlackboardScrollView.this.postDelayed(scrollerTask, newCheckTime);
            }
        };

        gestureDetector = new GestureDetector(context, gestureListener);
    }

    @TargetApi(21)
    public BlackboardScrollView(Context context, AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
    }

    @Override
    public boolean onInterceptTouchEvent(MotionEvent event){

        if (forbidScroll)
            return false;

        if (event.getActionMasked() == MotionEvent.ACTION_POINTER_DOWN){
            doubleFingerConfirmed = true;
        }else if (event.getActionMasked() == MotionEvent.ACTION_POINTER_UP){
            doubleFingerConfirmed = false;
        }
        gestureDetector.onTouchEvent(event);
        if (isSingleFingerScrollEnable || doubleFingerConfirmed)
            return super.onInterceptTouchEvent(event);
        return false;
    }

    public void startScrollTask() {
        initialPosition = getScrollY();
        this.postDelayed(scrollerTask, newCheckTime);
    }

    @Override
    public boolean onTouchEvent(MotionEvent event){
        if (forbidScroll) return false;
        if (event.getAction() == MotionEvent.ACTION_UP){
            doubleFingerConfirmed = false;
            startScrollTask();
        }
        return super.onTouchEvent(event);
    }


    public interface ScrollViewListener {
        void onScrollChanged (ScrollView scrollView, int x, int y, int oldx, int oldy);

        void onScrollStopped (float ratio);
    }

    public void setScrollViewListener(ScrollViewListener scrollViewListener) {
        this.scrollViewListener = scrollViewListener;
    }

    public void setScrollEnable(boolean scrollEnable) {
        isSingleFingerScrollEnable = scrollEnable;
    }

    public void setForbidScroll() {
        forbidScroll = true;
    }

    public int getScrollViewHeight() {
        View view =  BlackboardScrollView.this.getChildAt(BlackboardScrollView.this.getChildCount() - 1);
        return (view.getBottom() - getHeight());
    }
}
